#!/opt/LifeKeeper/bin/perl
# Copyright (c) 2020, 2022 SIOS Technology Corp.
########################################################################
#
# Title:       restore for Load Balancer probe reply
#
#
# Usage:    restore -t tagname -i id
#
#           where:
#           "tagname" is the resource tag name
#           "id" is the resource ID
#              
# Logging:  All data sent to STDERR will be logged in the lifekeeper log file
#           at the INFO level.

use strict;
use Getopt::Std; # use the standard getopt package
BEGIN { require '/etc/default/LifeKeeper.pl'; } # use the LifeKeeper libraries
use LK;
use LKBase;
use IO::Socket::INET;
use vars qw($opt_t $opt_i);

my $thisCMD=`basename $0`;
chomp $thisCMD;
my $userinfo;
my $tag;
my $id;

my $HC_TIMEOUT = ($ENV{'HC_TIMEOUT'} =~ /^\d+$/) ? $ENV{'HC_TIMEOUT'} : 60;

sub parseOptions { # noparams

	getopts('t:i:');
	$tag = "$opt_t";
	$id = "$opt_i";
};

sub getInfo { # params: tag 

	my $tag = shift;
	@_ = LK::ins_list('', '', '', '', $tag, '');
	if ( @_ == 0 ){
		printf STDERR "$thisCMD: Internal error ins_list failed\n";
		exit 1;
	}
	@_ = split /\001/, $_[0];
	return ($_[5]);
};

sub check_exist_process {
	my $id = shift;
	my $pidfile = "$ENV{'LKROOT'}/config/$id.pid";
	if(open(my $PID, "< $pidfile")) {
		my $pid=<$PID>;
		close($PID);
		#
		# $perl expects to include full path of perl command like /opt/LifeKeeper/bin/perl.
		# $cmd expects to include the full path of Gen-LB restore script like
		# /opt/LifeKeeper/subsys/gen/resources/app/actions/!restore/$tag.
		#
		my ($perl, $cmd) = split / /, `ps $pid | awk '{if (\$1 == "$pid") print \$5, \$6}'`;
		if (defined $perl && $perl =~ /\/perl$/ && defined $cmd && $cmd =~/restore\/$tag$/) {
			LKBase::LogMsg({'STDERR' => 1}, LK::LK_INFO, "gen-lb", "restore", $tag, 127001, "Daemon already started.");
			exit 0;
		}
	}
}

#############################
# Main execution begins here
#############################

# parse command line options
parseOptions;

# $tag and $id are set now if they were given on the command line
($userinfo)=getInfo($tag);

check_exist_process($id);

(my $port, my $msg) = split(/ /, $userinfo);

if ($port !~ /^\d+$/ or $port < 1024 or $port > 65535) {
	LKBase::LogMsg({'STDERR' => 1}, LK::LK_ERROR, "gen-lb", "restore", $tag, 127012, "Port number is not valid.");
	exit 1;
}

my $socket = new IO::Socket::INET( Proto => "tcp",
				   LocalPort => $port,
				   Listen => 1,
				   Reuse => 1);
if (!defined $socket) {
	LKBase::LogMsg({'STDERR' => 1}, LK::LK_ERROR, "gen-lb", "restore", $tag, 127002, "Socket open failed.");
	exit 1;
}

# Wait for probe connection.
$SIG{ALRM} = sub {
	# If no probe, Possible configuration mistake.
	# In that case, it cannot be protected.
	LKBase::LogMsg({'STDERR' => 1}, LK::LK_ERROR, "gen-lb", "restore", $tag, 127003, "Health probe not received.");
	exit 1;
};
alarm $HC_TIMEOUT;
my $client = $socket->accept();
if (defined $msg) {
	my $request = <$client>;
	print $client $msg;
}
close $client;

# Probe reply daemon
alarm 0;
LK::flg_remove('', "$id-hc-ng");
LKBase::LogMsg({'LOGFILE' => 1}, LK::LK_DEBUG, "gen-lb", "restore", $tag, 127004, "Health check received.");

if (my $pid = fork) {
	# parent
	my $pidfile = "$ENV{'LKROOT'}/config/$id.pid";
	if(open(my $PID, "> $pidfile")) {
		print $PID "$pid";
		close($PID);
	}
} elsif (!defined $pid) {
	LKBase::LogMsg({'STDERR' => 1}, LK::LK_ERROR, "gen-lb", "restore", $tag, 127005, "Daemon stat failed");
} else {
	# child
	$SIG{ALRM} = sub {
		# If the probe does not come, there is a possibility that
		# the local node is isolated, so it is set to fail.
		LKBase::LogMsg({'LOGFILE' => 1}, LK::LK_ERROR, "gen-lb", "restore", $tag, 127006, "Health probe not received.");
		LK::flg_create('', "$id-hc-ng");
	};
	while(1) {
		alarm $HC_TIMEOUT;
		$client = $socket->accept();
		alarm 0;
		if (defined $client) {
			if (defined $msg) {
				my $request = <$client>;
				print $client $msg;
			}
			close($client);
			LKBase::LogMsg({'LOGFILE' => 1}, LK::LK_DEBUG, "gen-lb", "restore", $tag, 127007, "Health check received.");
			LK::flg_remove('', "$id-hc-ng");
		} else {
			LKBase::LogMsg({'LOGFILE' => 1}, LK::LK_ERROR, "gen-lb", "restore", $tag, 127011, "Failed to accept the health probe:$!");
		}
	}
}
exit 0;
